@@ -1,7 +1,7 @@
 use Modern::Perl;
 
 package DBIx::Squirrel;
-$DBIx::Squirrel::VERSION = '1.2.6'; # TRIAL
+$DBIx::Squirrel::VERSION = '1.3.0'; # TRIAL
 =pod
 
 =encoding UTF-8
@@ -12,7 +12,7 @@ DBIx::Squirrel - A C<DBI> extension
 
 =head1 VERSION
 
-version 1.2.6
+version 1.3.0
 
 =head1 SYNOPSIS
 
@@ -45,7 +45,7 @@ version 1.2.6
     db(DBIx::Squirrel->connect($dsn, $user, $pass, \%attr));
 
     # First, we need to associate "product" with a result set, then use
-    # "product" to reference itt in future. The next time arguments are
+    # "product" to reference it in future. The next time arguments are
     # passed, they are treated as bind-values when the statement is
     # executed.
     #
@@ -341,7 +341,6 @@ use DBIx::Squirrel::rs   ();
 use DBIx::Squirrel::rc   ();
 use DBIx::Squirrel::util qw/throw uniq/;
 
-
 BEGIN {
     @DBIx::Squirrel::ISA                          = 'DBI';
     *DBIx::Squirrel::EXPORT_OK                    = *DBI::EXPORT_OK;
@@ -353,20 +352,23 @@ BEGIN {
     *DBIx::Squirrel::state                        = *DBI::state;
     *DBIx::Squirrel::connect                      = *DBIx::Squirrel::dr::connect;
     *DBIx::Squirrel::connect_cached               = *DBIx::Squirrel::dr::connect_cached;
-    *DBIx::Squirrel::SQL_ABSTRACT                 = *DBIx::Squirrel::db::SQL_ABSTRACT;
+    *DBIx::Squirrel::FINISH_ACTIVE_BEFORE_EXECUTE = *DBIx::Squirrel::st::FINISH_ACTIVE_BEFORE_EXECUTE;
     *DBIx::Squirrel::DEFAULT_SLICE                = *DBIx::Squirrel::it::DEFAULT_SLICE;
     *DBIx::Squirrel::DEFAULT_MAXROWS              = *DBIx::Squirrel::it::DEFAULT_MAXROWS;
     *DBIx::Squirrel::BUF_MULT                     = *DBIx::Squirrel::it::BUF_MULT;
     *DBIx::Squirrel::BUF_MAXROWS                  = *DBIx::Squirrel::it::BUF_MAXROWS;
-    $DBIx::Squirrel::FINISH_ACTIVE_BEFORE_EXECUTE = !!1;
-    $DBIx::Squirrel::NORMALISE_SQL                = !!1;
-    *DBIx::Squirrel::NORMALIZE_SQL                = *DBIx::Squirrel::NORMALISE_SQL;
+    *DBIx::Squirrel::NORMALISE_SQL                = *DBIx::Squirrel::util::NORMALISE_SQL;
+    *DBIx::Squirrel::NORMALIZE_SQL                = *DBIx::Squirrel::util::NORMALISE_SQL;
+
+    unless (defined $DBIx::Squirrel::VERSION) {
+        my $v = "1.3.0";
+        *DBIx::Squirrel::VERSION = \$v;
+    }
 }
 
 use constant E_BAD_ENT_BIND     => 'Cannot associate with an invalid object';
 use constant E_EXP_HASH_ARR_REF => 'Expected a reference to a HASH or ARRAY';
 
-
 sub _partition_imports_into_helpers_and_dbi_imports {
     my $class = shift;
     my(@helpers, @dbi);
@@ -392,7 +394,6 @@ sub _partition_imports_into_helpers_and_dbi_imports {
     return (\@helpers, \@dbi);
 }
 
-
 sub import {
     no strict 'refs';    ## no critic
     my $class  = shift;
@@ -465,11 +466,11 @@ __END__
 
 =head1 DESCRIPTION
 
-The C<DBIx::Squirrel> package extends the C<DBI>, by providing a few extra
+The C<DBIx::Squirrel> package extends the C<DBI> by providing a few extra
 conveniences that are subtle and additive in nature, and, hopefully, quite
 useful.
 
-=head2 Importing the package
+=head2 IMPORTING
 
 In the simplest case, just import the package as you would any other:
 
@@ -527,9 +528,9 @@ Addressing an association amounts to doing something meaningful with it,
 and we accomplish this by calling the helper function with one or more
 arguments.
 
-Once associated with a database entity, a helper function will any arguments
-that are passed to it and send a version of these to the database entity
-method that imbues meaning to the interaction.
+Once associated with a database entity, a helper function will consume
+any arguments that are passed to it and send a version of these to the
+database entity method that imparts meaning to the interaction.
 
 Meaning in this context is determined by the type of association:
 
@@ -546,12 +547,13 @@ methods respectively.
 
 =back
 
-B<Clearly there is a paradox here>, which centres around those statements
-and iterators expecting I<no bind-values>. In order to smooth-out this wrinkle,
-you can opt to enclose arguments inside an anonymous array or hash. When no
-bind-values are expected, you can coerce the helper into performing the
-execution by passing an empty array or hash reference. Alternatively, you
-could just resolve the association and call the relevant method manually.
+B<Clearly there is a paradox here>, and itcentres around those statements
+and iterators that take no parameters and expect no bind-values. In order
+to smooth-out this wrinkle, you can opt to enclose arguments inside an
+anonymous array or hash. When no bind-values are expected, you can coerce
+the helper into performing the execution by passing an empty ARRAYREF or
+HASHREF. Alternatively, you could just resolve the association and call
+the relevant method manually.
 
 =back
 
@@ -600,7 +602,7 @@ to dip a toe in the water ahead of time:
 
 =back
 
-=head2 Connecting to databases
+=head2 CONNECTING TO DATABASES
 
 Connecting to a database using C<DBIx::Squirrel> may be done exactly as it
 would when using the C<DBI>'s C<connect_cached> and C<connect> methods.
@@ -612,12 +614,12 @@ an alternative form:
 
     $new_dbh = DBIx::Squirrel->connect($original_dbh, \%attr);
 
-This form clones another connection object and returns a brand object that
-is blessed using the same class that invoked the C<connect> method. Objects
-being cloned are allowed to be those created by the C<DBI> or any of its
-subclasses, C<DBIx::Squirrel> being one of those.
+This form clones another connection object and returns a brand new object
+that is blessed using the same class that invoked the C<connect> method.
+The method will allow you to clone database connections created by the
+C<DBI> and any subclasses (C<DBIx::Squirrel> being one).
 
-=head2 Preparing statements
+=head2 PREPARING STATEMENTS
 
 Preparing a statement using C<DBIx::Squirrel> may be done exactly as
 it would be done using the C<DBI>'s C<prepare_cached> and C<prepare>
@@ -627,8 +629,7 @@ methods.
 
 A nice quality-of-life improvement offered by C<DBIx::Squirrel>'s own
 implementation of the C<prepare_cached> and C<prepare> methods is their
-built-in ability to cope with for a number of different placeholder
-styles:
+built-in ability to cope with a number of different placeholder styles:
 
 =over
 
@@ -640,9 +641,9 @@ styles:
 
 =back
 
-It does not matter what style your C<DBD>-drive supports, C<DBIx::Squirrel>
+It does not matter what style your C<DBD>-driver supports, C<DBIx::Squirrel>
 will happily deal with all of the above styles. Just pick the one that
-you prefer to work with, or use the one that is most suitable to the
+you prefer to work with, or use the one that is most suitable for the
 task at hand.
 
 By the time your statement is passed to the C<DBD>-driver for execution,
@@ -707,7 +708,7 @@ Oracle named placeholders (C<:name>):
 
 =back
 
-=head2 Iterators
+=head2 ITERATORS
 
 In addition to statement objects, C<DBIx::Squirrel> provides two kinds
 of iterator:
@@ -722,13 +723,13 @@ of iterator:
 
 =head3 Basic Iterators
 
-Basic iterators present row data as ARRAYREFs or HASHREFs, depending
+Basic iterators present row data as ARRAYREFs or HASHREFs depending
 on the slice-style currently in use. Column values are accessed either
-by column-index when using the ARRAYREFs-slice-style, or by column-
-name when using the HASHREFs-slice-style.
+by column-index when using the ARRAYREF-slicing, or by column-name
+when using the HASHREF-slicing.
 
-The default slice-style is ARRAYREFs. An iterator's "reset" method may
-be used to alter this behaviour.
+The default, row data is sliced as an ARRAYREF. The iterator "reset"
+method may be used to alter this behaviour.
 
 =head4 How to create a basic iterator
 
@@ -772,7 +773,7 @@ created the first time they are used.
 The C<results> methods may be replaced by either of the C<rs> or C<resultset>
 aliases, if preferred.
 
-=head2 Transforming results
+=head2 TRANSFORMING RESULTS
 
 All C<DBIx::Squirrel> iterators support an optional processing step called
 I<transformation>.
@@ -814,7 +815,7 @@ rely on something like C<$_>, but would like something much less ephemeral,
 just C<use DBIx::Squirrel::util 'result'> and use the C<result> function
 inside your transformation stage.
 
-Hand-off to the next stage, or the caller, is via an explicit C<return>
+Handing off to the next stage, or the caller, is with an explicit C<return>
 statement, or the result of evaluating the unit's final expression. Returning
 nothingE<mdash>either C<()>, or a bare C<return>E<mdash>from a processing
 step will filter the result out entirely, and no further processing steps
@@ -826,7 +827,7 @@ will apply to it.
 
 =item 1.
 
-See script C<examples/transformations_1.pl>:
+See script C<examples/transformations/01.pl>:
 
     use DBIx::Squirrel database_entities => [qw/db get_artist_id_by_name/];
 
@@ -901,7 +902,7 @@ I am only intersted in getting back the artist's id.
 
 B<Query the database and process the results>
 
-Here, I'm executing the query once for each one of four artist to get and
+Here, I'm executing the query once for each one of four artists to get and
 print their artist ids.
 
 =item *
@@ -914,7 +915,7 @@ Just as we would with the C<DBI>.
 
 Find the script and run it:
 
-    $ perl -I./lib ./examples/transformations_1.pl
+    $ perl -Ilib examples/transformations/01.pl
     ----
     Name: AC/DC
     ArtistId: 1
@@ -926,11 +927,409 @@ Find the script and run it:
     ArtistId: 128
 
 Notice that we got nothing back for one of our artists? That's because
-the artist in question is not in our database, and we cannot apply a
-transformation to nothing.
+the artist in question is not in our database and we cannot apply a
+transformation to nothing, so nothing is returned.
 
 =back
 
+=head2 REFERENCE
+
+This section describes the C<DBIx::Squirrel> interface.
+
+Many of the methods (*) presented below may seem familiar to the experienced
+C<DBI> user, and they should. They are documented here because C<DBIx::Squirrel>
+makes subtle changes to their interfaces.
+
+Such changes are additive and unobtrusive in nature, in most cases, resulting
+in additional calling forms rather than any change in outcome. Unless a
+documented deviation from the standard C<DBI> behaviour exists, one may
+safely assume that the C<DBI> documentation still applies.
+
+Other parts of the C<DBI> interface remain unaltered, as well as being
+accessible via C<DBIx::Squirrel>.
+
+=head3 DBIx::Squirrel Class Methods
+
+=head4 C<connect> *
+
+    $dbh = DBIx::Squirrel->connect($data_source, $username, $password)
+                or die $DBIx::Squirrel::errstr;
+    $dbh = DBIx::Squirrel->connect($data_source, $username, $password, \%attr)
+                or die $DBIx::Squirrel::errstr;
+    $clone_dbh = DBIx::Squirrel->connect($dbh)
+                or die $DBIx::Squirrel::errstr;
+    $clone_dbh = DBIx::Squirrel->connect($dbh, \%attr)
+                or die $DBIx::Squirrel::errstr;
+
+=head4 C<connect_cached> *
+
+    $dbh = DBIx::Squirrel->connect_cached($data_source, $username, $password)
+                or die $DBIx::Squirrel::errstr;
+    $dbh = DBIx::Squirrel->connect_cached($data_source, $username, $password, \%attr)
+                or die $DBIx::Squirrel::errstr;
+
+=head3 Database Handle Methods
+
+=head4 C<do> *
+
+    $rows = $dbh->do($statement)
+                or die $dbh->errstr;
+    $rows = $dbh->do($statement, \%attr)
+                or die ...;
+    $rows = $dbh->do($statement, \%attr, @bind_values)
+                or die ...;
+    $rows = $dbh->do($statement, \%attr, %bind_mappings)
+                or die ...;
+    $rows = $dbh->do($statement, \%attr, \@bind_values)
+                or die ...;
+    $rows = $dbh->do($statement, \%attr, \%bind_mappings)
+                or die ...;
+    $rows = $dbh->do($statement, @bind_values)
+                or die ...;
+    $rows = $dbh->do($statement, %bind_mappings)
+                or die ...;
+    $rows = $dbh->do($statement, \@bind_values)
+                or die ...;
+    $rows = $dbh->do($statement, undef, \%bind_mappings)
+                or die ...;
+
+Calling C<do> in scalar-context works just as it does when using the C<DBI>,
+although there are a few more calling forms.
+
+Calling C<do> in list-context, however, is new behaviour and results in the
+return of a list comprised of two elements: the number of rows affected by
+the statement, as well as the statement handle:
+
+    ($rows, $sth) = $dbh->do($statement)
+                or die $dbh->errstr;
+    ($rows, $sth) = $dbh->do($statement, \%attr)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, \%attr, @bind_values)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, \%attr, %bind_mappings)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, \%attr, \@bind_values)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, \%attr, \%bind_mappings)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, @bind_values)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, %bind_mappings)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, \@bind_values)
+                or die ...;
+    ($rows, $sth) = $dbh->do($statement, undef, \%bind_mappings)
+                or die ...;
+
+=head4 C<iterate>
+
+    $itor = $dbh->iterate($statement)
+                or die $dbh->errstr;
+    $itor = $dbh->iterate($statement, @transforms)
+                or die $dbh->errstr;
+    $itor = $dbh->iterate($statement, \%attr)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, @transforms)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, @bind_values)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, @bind_values, @transforms)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, %bind_mappings)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, %bind_mappings, @transforms)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, \@bind_values)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, [@bind_values, @transforms])
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, \%bind_mappings)
+                or die ...;
+    $itor = $dbh->iterate($statement, \%attr, \%bind_mappings, @transforms)
+                or die ...;
+    $itor = $dbh->iterate($statement, @bind_values)
+                or die ...;
+    $itor = $dbh->iterate($statement, @bind_values, @transforms)
+                or die ...;
+    $itor = $dbh->iterate($statement, %bind_mappings)
+                or die ...;
+    $itor = $dbh->iterate($statement, %bind_mappings, @transforms)
+                or die ...;
+    $itor = $dbh->iterate($statement, \@bind_values)
+                or die ...;
+    $itor = $dbh->iterate($statement, [@bind_values, @transforms])
+                or die ...;
+    $itor = $dbh->iterate($statement, undef, \%bind_mappings)
+                or die ...;
+    $itor = $dbh->iterate($statement, undef, \%bind_mappings, @transforms)
+                or die ...;
+
+=head4 C<prepare> *
+
+    $sth = $dbh->prepare($statement)          or die $dbh->errstr;
+    $sth = $dbh->prepare($statement, \%attr)  or die $dbh->errstr;
+
+The C<prepare> method interface is identical in form to that provided by the
+C<DBI>.
+
+C<DBIx::Squirrel> permits the use of one of a number of valid placeholder
+styles (C<:name>, C<:number>, C<$number>, C<?number>, C<?>) within the
+statement-string.
+
+Statement-strings will be "normalised" to use the legacy C<?> style, before
+being handed-off to the C<DBI> method of the same name. In spite of this,
+you should still use key-value bindings if you opted for named placeholders.
+
+=head4 C<prepare_cached> *
+
+    $sth = $dbh->prepare_cached($statement)
+    $sth = $dbh->prepare_cached($statement, \%attr)
+    $sth = $dbh->prepare_cached($statement, \%attr, $if_active)
+
+The C<prepare_cached> method interface is identical in form to that provided
+by the C<DBI>.
+
+C<DBIx::Squirrel> permits the use of one of a number of valid placeholder
+styles (C<:name>, C<:number>, C<$number>, C<?number>, C<?>) within the
+statement-string.
+
+Statement-strings will be "normalised" to use the legacy C<?> style, before
+being handed-off to the C<DBI> method of the same name. In spite of this,
+you should still use key-value bindings if you opted for named placeholders.
+
+It is the normalised form of the statement that is cached by the C<DBI>.
+
+=head4 C<results>
+
+    $itor = $dbh->results($statement)
+                or die $dbh->errstr;
+    $itor = $dbh->results($statement, @transforms)
+                or die $dbh->errstr;
+    $itor = $dbh->results($statement, \%attr)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, @transforms)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, @bind_values)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, @bind_values, @transforms)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, %bind_mappings)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, %bind_mappings, @transforms)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, \@bind_values)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, [@bind_values, @transforms])
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, \%bind_mappings)
+                or die ...;
+    $itor = $dbh->results($statement, \%attr, \%bind_mappings, @transforms)
+                or die ...;
+    $itor = $dbh->results($statement, @bind_values)
+                or die ...;
+    $itor = $dbh->results($statement, @bind_values, @transforms)
+                or die ...;
+    $itor = $dbh->results($statement, %bind_mappings)
+                or die ...;
+    $itor = $dbh->results($statement, %bind_mappings, @transforms)
+                or die ...;
+    $itor = $dbh->results($statement, \@bind_values)
+                or die ...;
+    $itor = $dbh->results($statement, [@bind_values, @transforms])
+                or die ...;
+    $itor = $dbh->results($statement, undef, \%bind_mappings)
+                or die ...;
+    $itor = $dbh->results($statement, undef, \%bind_mappings, @transforms)
+                or die ...;
+
+=head3 Statement Handle Methods
+
+=head4 C<bind>
+
+    $sth->bind(@bind_values);
+    $sth->bind(\@bind_values);
+    $sth->bind(%bind_mappings);
+    $sth->bind(\%bind_mappings);
+
+=head4 C<bind_param> *
+
+    $sth->bind_param($p_num, $bind_value);
+    $sth->bind_param($p_num, $bind_value, \%attr);
+    $sth->bind_param($p_num, $bind_value, $bind_type);
+    $sth->bind_param($p_name, $bind_value);
+    $sth->bind_param($p_name, $bind_value, \%attr);
+    $sth->bind_param($p_name, $bind_value, $bind_type);
+
+=head4 C<execute> *
+
+    $rv = $sth->execute();
+    $rv = $sth->execute(@bind_values);
+    $rv = $sth->execute(\@bind_values);
+    $rv = $sth->execute(%bind_mappings);
+    $rv = $sth->execute(\%bind_mappings);
+
+=head4 C<iterate>
+
+    $itor = $sth->iterate()
+                or die $dbh->errstr;
+    $itor = $sth->iterate(@bind_values)
+                or die ...;
+    $itor = $sth->iterate(@transforms)
+                or die $dbh->errstr;
+    $itor = $sth->iterate(@bind_values, @transforms)
+                or die ...;
+    $itor = $sth->iterate(%bind_mappings)
+                or die ...;
+    $itor = $sth->iterate(%bind_mappings, @transforms)
+                or die ...;
+    $itor = $sth->iterate(\@bind_values)
+                or die ...;
+    $itor = $sth->iterate([@bind_values, @transforms])
+                or die ...;
+    $itor = $sth->iterate(\%bind_mappings)
+                or die ...;
+    $itor = $sth->iterate(\%bind_mappings, @transforms)
+                or die ...;
+
+=head4 C<results>
+
+    $itor = $sth->results()
+                or die $dbh->errstr;
+    $itor = $sth->results(@bind_values)
+                or die ...;
+    $itor = $sth->results(@transforms)
+                or die $dbh->errstr;
+    $itor = $sth->results(@bind_values, @transforms)
+                or die ...;
+    $itor = $sth->results(%bind_mappings)
+                or die ...;
+    $itor = $sth->results(%bind_mappings, @transforms)
+                or die ...;
+    $itor = $sth->results(\@bind_values)
+                or die ...;
+    $itor = $sth->results([@bind_values, @transforms])
+                or die ...;
+    $itor = $sth->results(\%bind_mappings)
+                or die ...;
+    $itor = $sth->results(\%bind_mappings, @transforms)
+                or die ...;
+
+=head3 Iterator Objects
+
+=head4 C<all>
+
+    @ary = $itor->all();
+    $ary_ref | undef = $itor->all();
+
+Executes the iterator's underlying statement handle object.
+
+When called in list-context, the C<all> method returns an array
+of all matching row objects.
+
+When called in scalar-context, this method returns a reference to
+an array of all matching row objects. Where no rows are matched,
+C<undef> would be returned.
+
+=head4 C<count>
+
+Alias I<(see C<countnext>)>.
+
+=head4 C<countall>
+
+    $count = $itor->countall();
+
+Returns the number of matching rows by calculating the size of the
+array that would be created when calling the C<all> method.
+
+Of the two counting methods, this may be the fastest but it is also
+the most memory-hungry.
+
+=head4 C<countnext>
+
+    $count = $itor->countnext();
+
+Returns the number of matching rows by counting the number of times
+C<next> can be called.
+
+Of the two counting methods, this may be the least memory-hungry.
+
+=head4 C<done>
+
+Alias I<(see C<finished>)>.
+
+=head4 C<execute>
+
+    $rv = $itor->execute());
+
+Executes the iterator's underlying statemeent handle object.
+
+=head4 C<executed>
+
+    $bool = $itor->executed();
+
+Returns true (C<!!1>) if the underlying statement has been executed,
+otherwise it returns false (C<!!0>).
+
+=head4 C<find>
+
+=head4 C<finish>
+
+=head4 C<finished>
+
+    $bool = $itor->finished();
+
+Returns true (C<!!1>) if all matching rows have been fetched,
+otherwise it returns false (C<!!0>).
+
+=head4 C<first>
+
+=head4 C<iterate>
+
+    $itor | undef = $itor->iterate());
+
+Executes the iterator's underlying statemeent handle object,
+returning the iterator object reference if execution was successful,
+or C<undef> if not.
+
+=head4 C<next>
+
+=head4 C<one>
+
+Alias I<(see C<single>)>.
+
+=head4 C<remaining>
+
+    @ary = $itor->remaining();
+    $ary_ref | undef = $itor->remaining();
+
+When called in list-context, the C<all> method returns an array
+of all matching row objects remaining to be fetched.
+
+When called in scalar-context, this method returns a reference to
+an array of all matching row objects remaining to be fetched. Where
+no rows are matched, C<undef> would be returned.
+
+=head4 C<reset>
+
+=head4 C<rows>
+
+    $rv = $itor->rows();
+
+Returns the number of rows aftected by non-SELECT statements.
+
+=head4 C<single>
+
+=head4 C<statement_handle>
+
+    $sth = $itor->statement_handle();
+
+Returns a reference to the statement handle object associated with
+the iterator.
+
+=head4 C<sth>
+
+Alias I<(see C<statement_handle>)>.
+
 =head1 COPYRIGHT AND LICENSE
 
 The DBIx::Squirrel module is Copyright (c) 2020-2014 Iain Campbell.
